@@ -1,6 +1,7 @@
 package t::StupidLoop;
 
-use strict;
+use v5.14;
+use warnings;
 use base qw( IO::Async::Loop );
 
 sub new { return bless {}, shift; }
